/* i2c_driver.c - i2c single master driver

Copyright (c) 2012 David Michael Betz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute,
sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include <propeller.h>
#include "i2c_driver.h"
#include "i2c_driver_lib.h"

/* i2c state information */
static _COGMEM volatile I2C_MAILBOX *mailbox;

_NAKED int main(void)
{
    I2C_INIT *init = (I2C_INIT *)PAR;
    I2C_CMD cmd;
    uint8_t *p;
    uint32_t count;
    
    /* get the COG initialization parameters */
    i2cInit(init->scl, init->sda, init->ticks_per_cycle);
    mailbox = init->mailbox;
    
    /* tell the caller that we're done with initialization */
    mailbox->cmd = I2C_CMD_IDLE;
    
    /* handle requests */
    for (;;) {
        uint32_t sts;
    
        /* wait for the next request */
        while ((cmd = mailbox->cmd) == I2C_CMD_IDLE)
            ;
        
        /* dispatch on the command code */
        switch (cmd) {
        case I2C_CMD_SEND:
            p = mailbox->buffer;
            count = mailbox->count;
            sts = I2C_OK;
            i2cStart();
            if (i2cSendByte(mailbox->hdr) != 0)
                sts = I2C_ERR_SEND_HDR;
            else {
                while (count > 0) {
                    if (i2cSendByte(*p++) != 0) {
                        sts = I2C_ERR_SEND;
                        break;
                    }
                    --count;
                }
            }
            if (mailbox->stop)
                i2cStop();
            break;
        case I2C_CMD_RECEIVE:
            p = mailbox->buffer;
            count = mailbox->count;
            sts = I2C_OK;
            i2cStart();
            if (i2cSendByte(mailbox->hdr) != 0)
                sts = I2C_ERR_RECEIVE_HDR;
            else {
                while (count > 0) {
                    int byte = i2cReceiveByte(count != 1);
                    if (byte < 0) {
                        sts = I2C_ERR_RECEIVE;
                        break;
                    }
                    *p++ = byte;
                    --count;
                }
            }
            if (mailbox->stop)
                i2cStop();
            break;
        default:
            sts = I2C_ERR_UNKNOWN_CMD;
            break;
        }
        
        mailbox->sts = sts;
        mailbox->cmd = I2C_CMD_IDLE;
    }
    
    return 0;
}
